// StatusMgr.h : header file for StatusMgr.cpp of Engine.dll
//

#if !defined(_STATUS_MANAGER_INCLUDED_)
#define _STATUS_MANAGER_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

typedef CString (* STATUSMGRHOOK) ();

/////////////////////////////////////////////////////////////////////////////
// Status management routines.
//
class DECL TStatusMgr
{
   ////////////////////////////////
   // Interface
   ////////////////////////////////
   public:
   
   // Constructors and destructor.
   TStatusMgr();
   virtual ~TStatusMgr() {};

   // Initialize status cache to empty.
   void Init();

   // Start status message collection.
   void Start();
   
   // Stop status message collection.
   void Stop();
   
   // Return current status.
   CString GetStatus() const;

   // Read and return the kernels' current reset count.
   int ReadResetCount() const;

   // Write the kernel's current reset count.
   void WriteResetCount(int iResetCount);

   // Number of message in status cache.
   const int m_iCacheSize;

   // Look for previous exception dumps and log details, if found. 
   // Returns true if an exception dump is found and logged. 
   bool LogExceptionDumps();

   STATUSMGRHOOK m_pTelemetryHook;

   ////////////////////////////////
   // Implementation
   ////////////////////////////////
   protected:
   friend void MessageHandler(LPVOID pInfo, int iLevel, LPCTSTR tstrMsg);

   // Methods used when processing exception dumps. 
   bool LogExceptionDump(const CString& strDumpFile);
   bool ReadBlock(HANDLE h, DWORD rva, void* pBlock, DWORD dwBlockSize);
   CString ReadString(HANDLE h, DWORD rva);
   CString ExcCodeToString(ULONG32 nCode);

   // Container for most recent status messages.
   CString m_strMsgCache[5];

   // Add a status message to the history cache.
   void AddStatusMessage(int iLevel, LPCTSTR tstrMsg);

   // Date-time and tick count capture of last app init.
   TTime m_tAppInit;
   INT64 m_iAppInit;
};


//The global status mgr object
extern DECL TStatusMgr StatusMgr;

#endif // !defined(_STATUS_MANAGER_INCLUDED_)
